﻿using System;
using System.Globalization;
using System.Threading;
using System.Threading.Tasks;

namespace NetCoreLocalization
{
    class Program
    {
        /// <summary>
        /// 测试Async和Await模式，是否会对Thread.CurrentThread.CurrentCulture和Thread.CurrentThread.CurrentUICulture产生影响
        /// </summary>
        static void AsyncAwaitThreadCulture()
        {
            //设置主线程的CurrentCulture和CurrentUICulture为语言ja-JP
            Thread.CurrentThread.CurrentCulture = new CultureInfo("ja-JP");
            Thread.CurrentThread.CurrentUICulture = new CultureInfo("ja-JP");

            Console.WriteLine($"Thread id:{Thread.CurrentThread.ManagedThreadId.ToString()}=>主线程的CurrentCulture是{Thread.CurrentThread.CurrentCulture.ToString()}, CurrentUICulture是{Thread.CurrentThread.CurrentUICulture.ToString()}");

            //通过Task来启动第一层线程
            Task.Run(async () =>
            {
                //将第一层线程的CurrentCulture和CurrentUICulture改为zh-CN
                //Thread.CurrentThread.CurrentCulture = new CultureInfo("zh-CN");
                //Thread.CurrentThread.CurrentUICulture = new CultureInfo("zh-CN");

                Console.WriteLine($"Thread id:{Thread.CurrentThread.ManagedThreadId.ToString()}=>第一层线程的CurrentCulture是{Thread.CurrentThread.CurrentCulture.ToString()}, CurrentUICulture是{Thread.CurrentThread.CurrentUICulture.ToString()}");

                //通过Task来启动第二层线程
                Task task = Task.Run(() =>
                {
                    Console.WriteLine($"Thread id:{Thread.CurrentThread.ManagedThreadId.ToString()}=>第二层线程的CurrentCulture是{Thread.CurrentThread.CurrentCulture.ToString()}, CurrentUICulture是{Thread.CurrentThread.CurrentUICulture.ToString()}");

                    //通过Thread来启动第三层线程
                    Thread th = new Thread(new ThreadStart(() =>
                    {

                        Thread.Sleep(3000);

                        Console.WriteLine($"Thread id:{Thread.CurrentThread.ManagedThreadId.ToString()}=>第三层线程的CurrentCulture是{Thread.CurrentThread.CurrentCulture.ToString()}, CurrentUICulture是{Thread.CurrentThread.CurrentUICulture.ToString()}");
                    }));

                    th.IsBackground = true;
                    th.Start();

                    th.Join();//阻塞第二层线程，直到第三层线程th结束
                });

                Thread.Sleep(1000);

                Console.WriteLine($"Thread id:{Thread.CurrentThread.ManagedThreadId.ToString()}=>await之前CurrentCulture是{Thread.CurrentThread.CurrentCulture.ToString()}, CurrentUICulture是{Thread.CurrentThread.CurrentUICulture.ToString()}");

                await task;//await，直到第二层线程结束

                Console.WriteLine($"Thread id:{Thread.CurrentThread.ManagedThreadId.ToString()}=>await之后CurrentCulture是{Thread.CurrentThread.CurrentCulture.ToString()}, CurrentUICulture是{Thread.CurrentThread.CurrentUICulture.ToString()}");

            }).Wait();//阻塞主线程，直到第一层线程执行完毕
        }

        static void Main(string[] args)
        {
            //-------------------设置主线程为zh-CN-------------------
            Thread.CurrentThread.CurrentCulture = new CultureInfo("zh-CN");
            Thread.CurrentThread.CurrentUICulture = new CultureInfo("zh-CN");

            Console.WriteLine($"Message为:{DemoResource.Message}");

            //-------------------设置主线程为ja-JP-------------------
            Thread.CurrentThread.CurrentCulture = new CultureInfo("ja-JP");
            Thread.CurrentThread.CurrentUICulture = new CultureInfo("ja-JP");

            Console.WriteLine($"Message为:{DemoResource.Message}");

            //-------------------设置主线程为fr-FR-------------------
            Thread.CurrentThread.CurrentCulture = new CultureInfo("fr-FR");
            Thread.CurrentThread.CurrentUICulture = new CultureInfo("fr-FR");

            Console.WriteLine($"Message为:{DemoResource.Message}");

            //-------------------设置主线程为ja-JP，但是DemoResource为zh-CN-------------------
            Thread.CurrentThread.CurrentCulture = new CultureInfo("ja-JP");
            Thread.CurrentThread.CurrentUICulture = new CultureInfo("ja-JP");

            DemoResource.Culture = new CultureInfo("zh-CN");

            Console.WriteLine($"Message为:{DemoResource.Message}");

            //-------------------测试Async和Await模式-------------------
            AsyncAwaitThreadCulture();

            Console.WriteLine("按任意键结束...");
            Console.ReadKey();
        }
    }
}
